/* Integrate the equations of motion of a satellite on a transfer
	 orbit from Earth to Mars, taking into consideration the gravity
	 of Earth, Mars, and the sun for all time t */

// Thomas Reppert 03/17/07

#include  <stdio.h>
#include <stdlib.h>
#include   <time.h>

// Declare the Sensitivity function
void Sensitivity(int, int, int, double*, double*, double*);

// Declare the initialization function
void InitSatVsSun(double, double*);

/* Note: All array math performed disregarding the zeroth index in
   an attempt to make the C code more compatible with Matlab */

int main()
{
	// Initialize the time step, initial time, and final times (s)
  int Step = 50, InitTime = 0, FinalTimeBegin = 18000000,
		FinalTimeDesired = 18000000, FinalTimeCurrent = 0;
	// Declare the initial heliocentric state vector
	double InitWrtSun[7];
	// Declare the final body-relative state vectors
	double FinalWrtSun[7], FinalWrtMars[7];
	// Initialize the loop counter and maximum
	int i = 1, N = 1;
	// Declare the continuation parameter
	double Alpha;
	// Initialize the initial Earth-relative parking altitude
	double InitAltitude = 7500.0;
	// Declare the CPU time count parameters
	double Time1, Time2, TimeDiff;
	// Declare file opening error flag
	//errno_t Error;
	// Declare pointers to files for writing
	//FILE* InitialStates;
	//FILE* FinalStates;

	// Define switches for enabling write to output file
	//#define INITIAL_STATES
	//#define FINAL_STATES

	#ifdef INITIAL_STATES
		// Check to make sure the output file opens correctly
		// If it doesn't open, leave the program
		if(Error = fopen_s(&InitialStates, "InitialStates.txt", "w") != 0)
		{ 
			printf("*** Output file could not be opened.\n");
			exit(EXIT_FAILURE);
		}

		// Prepare the InitialStates output file
		fprintf(InitialStates,"      t_f        Vx_0             Vy_0               VxP               VyP\n");
		fprintf(InitialStates,"--------------------------------------------------------------------------------\n");
	#endif

	#ifdef FINAL_STATES
		// Check to make sure the output file opens correctly
		// If it doesn't open, leave the program
		if(Error = fopen_s(&FinalStates, "FinalStates02.txt", "w") != 0)
		{ 
			printf("*** Output file could not be opened.\n");
			exit(EXIT_FAILURE);
		}

		// Prepare the FinalStates output file
		fprintf(FinalStates,"      t_f       r_f          vr_f\n");
		fprintf(FinalStates,"-------------------------------------\n");
	#endif

	// Perform the heliocentric state vector initialization
	InitSatVsSun(InitAltitude, InitWrtSun);
	InitWrtSun[4] = 9.2842514189432173; InitWrtSun[5] = 35.235636452335747;

	// Begin the CPU time counter
	Time1 = clock();

	// Perform the continuation method loop
	while(i <= N)
	{
		// Calculate the continuation parameter
		Alpha = (double)i/N;

		// Calculate the current integration time interval
		FinalTimeCurrent = (int)(Alpha*FinalTimeDesired +
			(1.0 - Alpha)*FinalTimeBegin);

		#ifdef INITIAL_STATES
			// Print the current initial state vector components
			fprintf(InitialStates,"%9i %15.10e %15.10e ",FinalTimeCurrent,InitWrtSun[4],InitWrtSun[5]);
		#endif
		
		// Solve the two-point BVP for the current time interval
		Sensitivity(InitTime,FinalTimeCurrent,Step, InitWrtSun,FinalWrtSun,
			FinalWrtMars);

		#ifdef INITIAL_STATES
			// Print the corrected initial state vector components
			fprintf(InitialStates,"%15.10e %15.10e\n",InitWrtSun[4],InitWrtSun[5]);
		#endif

		#ifdef FINAL_STATES
			// Print the final state vector components
			fprintf(FinalStates,"%9i %12.7e %12.7e\n",FinalTimeCurrent,FinalWrtSun[1],FinalWrtSun[2]);
		#endif

		// Increment the loop counter
		i++;

		// Break after solving the BVP for the desired time interval
		if(i > N) break;
	}

	// End the CPU time counter
	Time2 = clock();
	TimeDiff = (Time2 - Time1)/CLOCKS_PER_SEC;
	//printf("Total computational time = %8.4f s.\n",TimeDiff);

	#ifdef INITIAL_STATES
		fclose(InitialStates);
	#endif

	#ifdef FINAL_STATES
		fclose(FinalStates);
	#endif

	return 0;
}
